@@ -1,10 +1,9 @@
 ## Tests for edge case one covariate and one Tree in Forest
-
 set.seed(1)
 n <- 10
 X <- matrix(rnorm(n * 1), nrow = n)
 y <- sign(X[, 1]) * 3 + rnorm(n)
-model <- SDForest(x = X, y = y, Q_type = 'no_deconfounding', cp = 0.1, nTree = 1)
+model <- SDForest(x = X, y = y, Q_type = 'no_deconfounding', cp = 0, nTree = 1, min_sample = 1)
 
 # does varImp work
 expect_equal(model$var_importance, varImp(model))
@@ -34,3 +33,10 @@ oob32 <- oob1 + oob2
 oob32[oob1 != 0 & oob2 != 0] <- oob32[oob1 != 0 & oob2 != 0] / 2
 
 expect_equal(oob3, oob32)
+
+#estimate Random Forests with only value in X
+y <- matrix(1, nrow = 50)
+X <- matrix(1, nrow = 50, ncol = 10)
+expect_no_error(SDForest(x = X, y = y, Q_type = "no_deconfounding", nTree = 10))
+X <- matrix(1, nrow = 50, ncol = 1)
+expect_no_error(SDForest(x = X, y = y, Q_type = "no_deconfounding", nTree = 10))
